@@ -15,9 +15,7 @@
 package proxy_test
 
 import (
-	"fmt"
 	"net"
-	"net/http"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -110,21 +108,9 @@ var _ = Describe("BPF kube-proxy", func() {
 			}).Should(BeTrue())
 		})
 
-		By("checking that the healthCheckNodePort is accessible", func() {
-			Eventually(func() error {
-				result, err := http.Get(fmt.Sprintf("http://localhost:%d", healthCheckNodePort))
-				if err != nil {
-					return err
-				}
-				if result.StatusCode != 503 {
-					return fmt.Errorf("Unexpected status code %d; expected 503", result.StatusCode)
-				}
-				return nil
-			}, "5s", "200ms").Should(Succeed())
-		})
+		updatedIP := net.IPv4(2, 2, 2, 2)
 
 		By("checking nodeport has the updated IP and not the initial IP", func() {
-			updatedIP := net.IPv4(2, 2, 2, 2)
 			p.OnHostIPsUpdate([]net.IP{updatedIP})
 
 			Eventually(func() bool {
@@ -143,19 +129,6 @@ var _ = Describe("BPF kube-proxy", func() {
 			}).Should(BeTrue())
 		})
 
-		By("checking that the healthCheckNodePort is still accessible", func() {
-			Eventually(func() error {
-				result, err := http.Get(fmt.Sprintf("http://localhost:%d", healthCheckNodePort))
-				if err != nil {
-					return err
-				}
-				if result.StatusCode != 503 {
-					return fmt.Errorf("Unexpected status code %d; expected 503", result.StatusCode)
-				}
-				return nil
-			}, "5s", "200ms").Should(Succeed())
-		})
-
 		By("checking nodeport has 2 updated IPs", func() {
 			ip1 := net.IPv4(3, 3, 3, 3)
 			ip2 := net.IPv4(4, 4, 4, 4)
