@@ -25,7 +25,6 @@ import (
 	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/apimachinery/pkg/util/sets"
 	k8sp "k8s.io/kubernetes/pkg/proxy"
 
 	"github.com/projectcalico/calico/felix/bpf"
@@ -45,10 +44,11 @@ func init() {
 
 var _ = Describe("BPF Syncer", func() {
 	var (
-		svcs *mockNATMap
-		eps  *mockNATBackendMap
-		aff  *mockAffinityMap
-		ct   *mock.Map
+		svcs      *mockNATMap
+		eps       *mockNATBackendMap
+		aff       *mockAffinityMap
+		ct        *mock.Map
+		ctCleanup *mock.Map
 
 		s        *proxy.Syncer
 		connScan *conntrack.Scanner
@@ -70,6 +70,7 @@ var _ = Describe("BPF Syncer", func() {
 		eps = newMockNATBackendMap()
 		aff = newMockAffinityMap()
 		ct = mock.NewMockMap(conntrack.MapParams)
+		ctCleanup = mock.NewMockMap(conntrack.MapParamsCleanup)
 
 		rt = proxy.NewRTCache()
 
@@ -171,7 +172,7 @@ var _ = Describe("BPF Syncer", func() {
 
 		By("creating a CT scanner", func() {
 			connScan = conntrack.NewScanner(ct,
-				conntrack.KeyFromBytes, conntrack.ValueFromBytes, conntrack.NewStaleNATScanner(s))
+				conntrack.KeyFromBytes, conntrack.ValueFromBytes, nil, "Disabled", ctCleanup, 4, mock.NewMockBPFCleaner(ct, ctCleanup), conntrack.NewStaleNATScanner(s))
 		})
 
 		By("creating conntrack entries for test-service", makestep(func() {
@@ -1090,7 +1091,7 @@ var _ = Describe("BPF Syncer", func() {
 
 		By("recreating a CT scanner for the actual syncer", func() {
 			connScan = conntrack.NewScanner(ct,
-				conntrack.KeyFromBytes, conntrack.ValueFromBytes, conntrack.NewStaleNATScanner(s))
+				conntrack.KeyFromBytes, conntrack.ValueFromBytes, nil, "Disabled", ctCleanup, 4, mock.NewMockBPFCleaner(ct, ctCleanup), conntrack.NewStaleNATScanner(s))
 		})
 
 		By("checking that CT table emptied by connScan", makestep(func() {
@@ -1112,95 +1113,6 @@ var _ = Describe("BPF Syncer", func() {
 			Expect(cnt).To(Equal(0))
 		}))
 
-		By("checking topology aware hints auto in service with multiple endpoints match first zone", makestep(func() {
-			state.SvcMap[svcKey] = proxy.NewK8sServicePort(
-				net.IPv4(10, 0, 0, 1),
-				1234,
-				v1.ProtocolTCP,
-				proxy.K8sSvcWithHintsAnnotation("auto"),
-			)
-			state.EpsMap[svcKey] = []k8sp.Endpoint{
-				proxy.NewEndpointInfo("10.1.0.1", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2a"))),
-				proxy.NewEndpointInfo("10.2.0.2", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2b"))),
-			}
-			state.NodeZone = "us-west-2a"
-
-			err := s.Apply(state)
-			Expect(err).NotTo(HaveOccurred())
-			Expect(eps.m).To(HaveLen(1))
-		}))
-
-		By("checking topology aware hints auto in service with multiple endpoints match second zone", makestep(func() {
-			state.SvcMap[svcKey] = proxy.NewK8sServicePort(
-				net.IPv4(10, 0, 0, 1),
-				1234,
-				v1.ProtocolTCP,
-				proxy.K8sSvcWithHintsAnnotation("auto"),
-			)
-			state.EpsMap[svcKey] = []k8sp.Endpoint{
-				proxy.NewEndpointInfo("10.1.0.1", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2a"))),
-				proxy.NewEndpointInfo("10.2.0.2", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2b"))),
-			}
-			state.NodeZone = "us-west-2b"
-
-			err := s.Apply(state)
-			Expect(err).NotTo(HaveOccurred())
-			Expect(eps.m).To(HaveLen(1))
-		}))
-
-		By("checking topology aware hints disabled in service with multiple endpoints match all zones", makestep(func() {
-			state.SvcMap[svcKey] = proxy.NewK8sServicePort(
-				net.IPv4(10, 0, 0, 1),
-				1234,
-				v1.ProtocolTCP,
-				proxy.K8sSvcWithHintsAnnotation("disabled"),
-			)
-			state.EpsMap[svcKey] = []k8sp.Endpoint{
-				proxy.NewEndpointInfo("10.1.0.1", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2a"))),
-				proxy.NewEndpointInfo("10.2.0.2", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2b"))),
-			}
-			state.NodeZone = "us-west-2b"
-
-			err := s.Apply(state)
-			Expect(err).NotTo(HaveOccurred())
-			Expect(eps.m).To(HaveLen(2))
-		}))
-
-		By("checking topology aware hints empty in service with multiple endpoints match all zones", makestep(func() {
-			state.SvcMap[svcKey] = proxy.NewK8sServicePort(
-				net.IPv4(10, 0, 0, 1),
-				1234,
-				v1.ProtocolTCP,
-			)
-			state.EpsMap[svcKey] = []k8sp.Endpoint{
-				proxy.NewEndpointInfo("10.1.0.1", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2a"))),
-				proxy.NewEndpointInfo("10.2.0.2", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2b"))),
-			}
-			state.NodeZone = "us-west-2b"
-
-			err := s.Apply(state)
-			Expect(err).NotTo(HaveOccurred())
-			Expect(eps.m).To(HaveLen(2))
-		}))
-
-		By("checking topology aware hints auto in service with multiple endpoints without node zone match all zones", makestep(func() {
-			state.SvcMap[svcKey] = proxy.NewK8sServicePort(
-				net.IPv4(10, 0, 0, 1),
-				1234,
-				v1.ProtocolTCP,
-				proxy.K8sSvcWithHintsAnnotation("auto"),
-			)
-			state.EpsMap[svcKey] = []k8sp.Endpoint{
-				proxy.NewEndpointInfo("10.1.0.1", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2a"))),
-				proxy.NewEndpointInfo("10.2.0.2", 5555, proxy.EndpointInfoOptIsReady(true), proxy.EndpointInfoOptZoneHints(sets.New[string]("us-west-2b"))),
-			}
-			state.NodeZone = ""
-
-			err := s.Apply(state)
-			Expect(err).NotTo(HaveOccurred())
-			Expect(eps.m).To(HaveLen(2))
-		}))
-
 		By("checking endpointslice terminating status should be included in endpointslice collection for processing", makestep(func() {
 
 			// Clean up all prior state.
