@@ -45,6 +45,7 @@ import (
 	"github.com/projectcalico/calico/felix/bpf/failsafes"
 	"github.com/projectcalico/calico/felix/bpf/hook"
 	"github.com/projectcalico/calico/felix/bpf/ifstate"
+	"github.com/projectcalico/calico/felix/bpf/ipfrags"
 	"github.com/projectcalico/calico/felix/bpf/ipsets"
 	"github.com/projectcalico/calico/felix/bpf/jump"
 	"github.com/projectcalico/calico/felix/bpf/libbpf"
@@ -53,6 +54,7 @@ import (
 	"github.com/projectcalico/calico/felix/bpf/perf"
 	"github.com/projectcalico/calico/felix/bpf/polprog"
 	"github.com/projectcalico/calico/felix/bpf/profiling"
+	"github.com/projectcalico/calico/felix/bpf/qos"
 	"github.com/projectcalico/calico/felix/bpf/routes"
 	"github.com/projectcalico/calico/felix/bpf/state"
 	tcdefs "github.com/projectcalico/calico/felix/bpf/tc/defs"
@@ -211,6 +213,7 @@ var tcJumpMapIndexes = map[string][]int{
 		tcdefs.ProgIndexHostCtConflict,
 		tcdefs.ProgIndexIcmpInnerNat,
 		tcdefs.ProgIndexNewFlow,
+		tcdefs.ProgIndexIPFrag,
 	},
 	"IPv4 debug": []int{
 		tcdefs.ProgIndexMainDebug,
@@ -221,6 +224,7 @@ var tcJumpMapIndexes = map[string][]int{
 		tcdefs.ProgIndexHostCtConflictDebug,
 		tcdefs.ProgIndexIcmpInnerNatDebug,
 		tcdefs.ProgIndexNewFlowDebug,
+		tcdefs.ProgIndexIPFragDebug,
 	},
 	"IPv6": []int{
 		tcdefs.ProgIndexMain,
@@ -306,6 +310,7 @@ func setupAndRun(logger testLogger, loglevel, section string, rules *polprog.Rul
 		logLevel:  log.DebugLevel,
 		psnaStart: 20000,
 		psnatEnd:  30000,
+		dscp:      -1,
 	}
 
 	for _, o := range opts {
@@ -476,8 +481,11 @@ func setupAndRun(logger testLogger, loglevel, section string, rules *polprog.Rul
 	}
 
 	if !topts.xdp {
+		_ = counters.EnsureExists(countersMap, 1, hook.Ingress)
+		_ = counters.EnsureExists(countersMap, 1, hook.Egress)
 		runFn(bpfFsDir + "/cali_tc_preamble")
 	} else {
+		_ = counters.EnsureExists(countersMap, 1, hook.XDP)
 		runFn(bpfFsDir + "/cali_xdp_preamble")
 	}
 }
@@ -576,12 +584,13 @@ func bpftool(args ...string) ([]byte, error) {
 var (
 	mapInitOnce sync.Once
 
-	natMap, natBEMap, ctMap, rtMap, ipsMap, testStateMap, affinityMap, arpMap, fsafeMap     maps.Map
-	natMapV6, natBEMapV6, ctMapV6, rtMapV6, ipsMapV6, affinityMapV6, arpMapV6, fsafeMapV6   maps.Map
-	stateMap, countersMap, ifstateMap, progMap, progMapXDP, policyJumpMap, policyJumpMapXDP maps.Map
-	perfMap                                                                                 maps.Map
-	profilingMap                                                                            maps.Map
-	allMaps                                                                                 []maps.Map
+	natMap, natBEMap, ctMap, ctCleanupMap, rtMap, ipsMap, testStateMap, affinityMap, arpMap, fsafeMap, ipfragsMap maps.Map
+	natMapV6, natBEMapV6, ctMapV6, ctCleanupMapV6, rtMapV6, ipsMapV6, affinityMapV6, arpMapV6, fsafeMapV6         maps.Map
+	stateMap, countersMap, ifstateMap, progMap, progMapXDP, policyJumpMap, policyJumpMapXDP                       maps.Map
+	perfMap                                                                                                       maps.Map
+	profilingMap, ipfragsMapTmp, ctlbProgsMap                                                                     maps.Map
+	qosMap                                                                                                        maps.Map
+	allMaps                                                                                                       []maps.Map
 )
 
 func initMapsOnce() {
@@ -592,6 +601,8 @@ func initMapsOnce() {
 		natBEMapV6 = nat.BackendMapV6()
 		ctMap = conntrack.Map()
 		ctMapV6 = conntrack.MapV6()
+		ctCleanupMap = conntrack.CleanupMap()
+		ctCleanupMapV6 = conntrack.CleanupMapV6()
 		rtMap = routes.Map()
 		rtMapV6 = routes.MapV6()
 		ipsMap = ipsets.Map()
@@ -605,17 +616,21 @@ func initMapsOnce() {
 		fsafeMap = failsafes.Map()
 		fsafeMapV6 = failsafes.MapV6()
 		countersMap = counters.Map()
+		ipfragsMap = ipfrags.Map()
+		ipfragsMapTmp = ipfrags.MapTmp()
 		ifstateMap = ifstate.Map()
 		policyJumpMap = jump.Map()
 		policyJumpMapXDP = jump.XDPMap()
 		profilingMap = profiling.Map()
+		ctlbProgsMap = nat.ProgramsMap()
+		qosMap = qos.Map()
 
 		perfMap = perf.Map("perf_evnt", 512)
 
-		allMaps = []maps.Map{natMap, natBEMap, natMapV6, natBEMapV6, ctMap, ctMapV6, rtMap, rtMapV6, ipsMap, ipsMapV6,
+		allMaps = []maps.Map{natMap, natBEMap, natMapV6, natBEMapV6, ctMap, ctMapV6, ctCleanupMap, ctCleanupMapV6, rtMap, rtMapV6, ipsMap, ipsMapV6,
 			stateMap, testStateMap, affinityMap, affinityMapV6, arpMap, arpMapV6, fsafeMap, fsafeMapV6,
-			countersMap, ifstateMap, profilingMap,
-			policyJumpMap, policyJumpMapXDP}
+			countersMap, ipfragsMap, ipfragsMapTmp, ifstateMap, profilingMap,
+			policyJumpMap, policyJumpMapXDP, ctlbProgsMap, qosMap}
 		for _, m := range allMaps {
 			err := m.EnsureExists()
 			if err != nil {
@@ -639,7 +654,7 @@ func cleanUpMaps() {
 	defer log.SetLevel(logLevel)
 
 	for _, m := range allMaps {
-		if m == stateMap || m == testStateMap || m == progMap || m == countersMap {
+		if m == stateMap || m == testStateMap || m == progMap || m == countersMap || m == ipfragsMapTmp {
 			continue // Can't clean up array maps
 		}
 		log.WithField("map", m.GetName()).Info("Cleaning")
@@ -748,7 +763,10 @@ func objLoad(fname, bpfFsDir, ipFamily string, topts testOpts, polProg, hasHostC
 
 	for m, err := obj.FirstMap(); m != nil && err == nil; m, err = m.NextMap() {
 		if m.IsMapInternal() {
-			if strings.HasPrefix(m.Name(), ".rodata") {
+			if ipFamily != "preamble" {
+				continue
+			}
+			if !strings.HasSuffix(m.Name(), ".rodata") {
 				continue
 			}
 			if forXDP {
@@ -780,6 +798,23 @@ func objLoad(fname, bpfFsDir, ipFamily string, topts testOpts, polProg, hasHostC
 				if topts.flowLogsEnabled {
 					globals.Flags |= libbpf.GlobalsFlowLogsEnabled
 				}
+				if topts.natOutExcludeHosts {
+					globals.Flags |= libbpf.GlobalsNATOutgoingExcludeHosts
+				}
+
+				if topts.ingressQoSPacketRate {
+					globals.Flags |= libbpf.GlobalsIngressPacketRateConfigured
+				}
+
+				if topts.egressQoSPacketRate {
+					globals.Flags |= libbpf.GlobalsEgressPacketRateConfigured
+				}
+
+				globals.DSCP = -1
+				if topts.dscp >= 0 {
+					globals.DSCP = topts.dscp
+				}
+
 				if topts.ipv6 {
 					copy(globals.HostTunnelIPv6[:], node1tunIPV6.To16())
 					copy(globals.HostIPv6[:], hostIP.To16())
@@ -813,12 +848,18 @@ func objLoad(fname, bpfFsDir, ipFamily string, topts testOpts, polProg, hasHostC
 			"key size":   m.KeySize(),
 			"value size": m.ValueSize(),
 		}).Debug("Pinning map")
-		cmd := exec.Command("bpftool", "map", "show", "pinned", pin)
-		log.WithField("cmd", cmd.String()).Debugf("executing")
-		out, _ := cmd.Output()
-		log.WithField("output", string(out)).Debug("map")
-		log.WithField("size", m.MaxEntries()).Debug("libbpf map")
-		log.WithField("entry size", m.ValueSize()).Debug("libbpf map")
+		fd, err := maps.GetMapFDByPin(pin)
+		if err != nil {
+			log.WithError(err).Debug("error getting map FD by pin")
+		} else {
+			mapInfo, err := maps.GetMapInfo(fd)
+			if err != nil {
+				log.WithError(err).Debug("error getting mapInfo by FD")
+			} else {
+				log.WithFields(log.Fields{"Type": mapInfo.Type, "MaxEntries": mapInfo.MaxEntries, "ValueSize": mapInfo.ValueSize, "KeySize": mapInfo.KeySize}).Debug("existing map")
+			}
+		}
+		log.WithFields(log.Fields{"Type": m.Type(), "MaxEntries": m.MaxEntries(), "ValueSize": m.ValueSize(), "KeySize": m.KeySize()}).Debug("new map")
 		if err := m.SetPinPath(pin); err != nil {
 			obj.Close()
 			return nil, fmt.Errorf("error pinning map %s: %w", m.Name(), err)
@@ -893,6 +934,9 @@ func objUTLoad(fname, bpfFsDir, ipFamily string, topts testOpts, polProg, hasHos
 
 	for m, err := obj.FirstMap(); m != nil && err == nil; m, err = m.NextMap() {
 		if m.IsMapInternal() {
+			if !strings.HasSuffix(m.Name(), ".rodata") {
+				continue
+			}
 			globals := libbpf.TcGlobalData{
 				Tmtu:       natTunnelMTU,
 				VxlanPort:  testVxlanPort,
@@ -917,11 +961,18 @@ func objUTLoad(fname, bpfFsDir, ipFamily string, topts testOpts, polProg, hasHos
 		}
 		pin := "/sys/fs/bpf/tc/globals/" + m.Name()
 		log.WithField("pin", pin).Debug("Pinning map")
-		cmd := exec.Command("bpftool", "map", "show", "pinned", pin)
-		log.WithField("cmd", cmd.String()).Debugf("executing")
-		out, _ := cmd.Output()
-		log.WithField("output", string(out)).Debug("map")
-		log.WithField("size", m.MaxEntries()).Debug("libbpf map")
+		fd, err := maps.GetMapFDByPin(pin)
+		if err != nil {
+			log.WithError(err).Debug("error getting map FD by pin")
+		} else {
+			mapInfo, err := maps.GetMapInfo(fd)
+			if err != nil {
+				log.WithError(err).Debug("error getting mapInfo by FD")
+			} else {
+				log.WithFields(log.Fields{"Type": mapInfo.Type, "MaxEntries": mapInfo.MaxEntries, "ValueSize": mapInfo.ValueSize, "KeySize": mapInfo.KeySize}).Debug("existing map")
+			}
+		}
+		log.WithFields(log.Fields{"Type": m.Type(), "MaxEntries": m.MaxEntries(), "ValueSize": m.ValueSize(), "KeySize": m.KeySize()}).Debug("new map")
 		if err := m.SetPinPath(pin); err != nil {
 			obj.Close()
 			return nil, fmt.Errorf("error pinning map %s: %w", m.Name(), err)
@@ -1095,6 +1146,9 @@ func runBpfUnitTest(t *testing.T, source string, testFn func(bpfProgRunFn), opts
 
 	ctxIn := make([]byte, 18*4)
 
+	_ = counters.EnsureExists(countersMap, 1, hook.Ingress)
+	_ = counters.EnsureExists(countersMap, 1, hook.Egress)
+
 	runTest := func() {
 		testFn(func(dataIn []byte) (bpfRunResult, error) {
 			res, err := bpftoolProgRun(bpfFsDir+"/unittest", dataIn, ctxIn)
@@ -1116,18 +1170,22 @@ func runBpfUnitTest(t *testing.T, source string, testFn func(bpfProgRunFn), opts
 }
 
 type testOpts struct {
-	description     string
-	subtests        bool
-	logLevel        log.Level
-	xdp             bool
-	psnaStart       uint32
-	psnatEnd        uint32
-	hostNetworked   bool
-	fromHost        bool
-	progLog         string
-	ipv6            bool
-	objname         string
-	flowLogsEnabled bool
+	description          string
+	subtests             bool
+	logLevel             log.Level
+	xdp                  bool
+	psnaStart            uint32
+	psnatEnd             uint32
+	hostNetworked        bool
+	fromHost             bool
+	progLog              string
+	ipv6                 bool
+	objname              string
+	flowLogsEnabled      bool
+	natOutExcludeHosts   bool
+	ingressQoSPacketRate bool
+	egressQoSPacketRate  bool
+	dscp                 int8
 }
 
 type testOption func(opts *testOpts)
@@ -1183,6 +1241,30 @@ func withFlowLogs() testOption {
 	}
 }
 
+func withNATOutExcludeHosts() testOption {
+	return func(o *testOpts) {
+		o.natOutExcludeHosts = true
+	}
+}
+
+func withIngressQoSPacketRate() testOption {
+	return func(o *testOpts) {
+		o.ingressQoSPacketRate = true
+	}
+}
+
+func withEgressQoSPacketRate() testOption {
+	return func(o *testOpts) {
+		o.egressQoSPacketRate = true
+	}
+}
+
+func withEgressDSCP(value int8) testOption {
+	return func(o *testOpts) {
+		o.dscp = value
+	}
+}
+
 func withObjName(name string) testOption {
 	return func(o *testOpts) {
 		o.objname = name
@@ -1407,6 +1489,10 @@ func resetRTMapV6(rtMap maps.Map) {
 	resetMap(rtMap)
 }
 
+func resetQoSMap(qosMap maps.Map) {
+	resetMap(qosMap)
+}
+
 func saveRTMap(rtMap maps.Map) routes.MapMem {
 	rt, err := routes.LoadMap(rtMap)
 	Expect(err).NotTo(HaveOccurred())
@@ -1488,8 +1574,8 @@ var ipv4Default = &layers.IPv4{
 	Protocol: layers.IPProtocolUDP,
 }
 
-var srcIPv6 = net.IP([]byte{0xff, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1})
-var dstIPv6 = net.IP([]byte{0xff, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2})
+var srcIPv6 = net.IP([]byte{0x20, 0x1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1})
+var dstIPv6 = net.IP([]byte{0x20, 0x1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2})
 var srcV6CIDR = ip.CIDRFromNetIP(srcIPv6).(ip.V6CIDR)
 var dstV6CIDR = ip.CIDRFromNetIP(dstIPv6).(ip.V6CIDR)
 
@@ -1872,6 +1958,7 @@ func resetBPFMaps() {
 	resetMap(fsafeMap)
 	resetMap(natMap)
 	resetMap(natBEMap)
+	resetMap(qosMap)
 }
 
 func TestMapIterWithDelete(t *testing.T) {
@@ -1928,7 +2015,7 @@ func TestMapIterWithDeleteLastOfBatch(t *testing.T) {
 		Type:       "hash",
 		KeySize:    8,
 		ValueSize:  8,
-		MaxEntries: 1000,
+		MaxEntries: 4 * maps.IteratorNumKeys,
 		Name:       "cali_tmap",
 		Flags:      unix.BPF_F_NO_PREALLOC,
 	})
@@ -1936,7 +2023,9 @@ func TestMapIterWithDeleteLastOfBatch(t *testing.T) {
 	err := m.EnsureExists()
 	Expect(err).NotTo(HaveOccurred())
 
-	for i := 0; i < 40; i++ {
+	items := 3*maps.IteratorNumKeys + 5
+
+	for i := 0; i < items; i++ {
 		var k, v [8]byte
 
 		binary.LittleEndian.PutUint64(k[:], uint64(i))
@@ -1965,9 +2054,10 @@ func TestMapIterWithDeleteLastOfBatch(t *testing.T) {
 	})
 	Expect(err).NotTo(HaveOccurred())
 
-	Expect(cnt).To(Equal(40))
+	Expect(len(out)).To(Equal(items))
+	Expect(cnt).To(Equal(items))
 
-	for i := 0; i < 40; i++ {
+	for i := 0; i < items; i++ {
 		Expect(out).To(HaveKey(uint64(i)))
 		Expect(out[uint64(i)]).To(Equal(uint64(i * 7)))
 	}
@@ -1976,6 +2066,10 @@ func TestMapIterWithDeleteLastOfBatch(t *testing.T) {
 func TestJumpMap(t *testing.T) {
 	RegisterTestingT(t)
 
+	progMap = hook.NewProgramsMap()
+	err := progMap.EnsureExists()
+	Expect(err).NotTo(HaveOccurred())
+
 	jumpMapFD := progMap.MapFD()
 	pg := polprog.NewBuilder(idalloc.New(), ipsMap.MapFD(), stateMap.MapFD(), jumpMapFD, policyJumpMap.MapFD(),
 		polprog.WithAllowDenyJumps(tcdefs.ProgIndexAllowed, tcdefs.ProgIndexDrop))
