@@ -30,6 +30,7 @@ import (
 	. "github.com/onsi/gomega"
 	log "github.com/sirupsen/logrus"
 	"github.com/vishvananda/netlink"
+	"golang.org/x/sys/unix"
 	googleproto "google.golang.org/protobuf/proto"
 
 	"github.com/projectcalico/calico/felix/bpf"
@@ -45,6 +46,7 @@ import (
 	bpfmaps "github.com/projectcalico/calico/felix/bpf/maps"
 	"github.com/projectcalico/calico/felix/bpf/mock"
 	"github.com/projectcalico/calico/felix/bpf/polprog"
+	"github.com/projectcalico/calico/felix/bpf/qos"
 	"github.com/projectcalico/calico/felix/bpf/state"
 	"github.com/projectcalico/calico/felix/bpf/tc"
 	"github.com/projectcalico/calico/felix/bpf/xdp"
@@ -77,6 +79,9 @@ type mockDataplane struct {
 	ensureStartedFn    func()
 	ensureQdiscFn      func(string) (bool, error)
 	interfaceByIndexFn func(ifindex int) (*net.Interface, error)
+
+	jitHarden             bool
+	finalTrampolineStride int
 }
 
 func newMockDataplane() *mockDataplane {
@@ -122,20 +127,18 @@ func (m *mockDataplane) loadDefaultPolicies() error {
 	return nil
 }
 
-func (m *mockDataplane) ensureProgramAttached(ap attachPoint) (qDiscInfo, error) {
+func (m *mockDataplane) ensureProgramAttached(ap attachPoint) error {
 	m.mutex.Lock()
 	defer m.mutex.Unlock()
 	key := ap.IfaceName() + ":" + ap.HookName().String()
 	m.numAttaches[key] = m.numAttaches[key] + 1
-	return qDiscInfo{valid: true, prio: 49152, handle: 1}, nil
+	return nil
 }
 
 func (m *mockDataplane) ensureProgramLoaded(ap attachPoint, ipFamily proto.IPVersion) error {
 	m.mutex.Lock()
 	defer m.mutex.Unlock()
 
-	// var res tc.AttachResult // we don't care about the values
-
 	if apxdp, ok := ap.(*xdp.AttachPoint); ok {
 		apxdp.HookLayoutV4 = hook.Layout{
 			hook.SubProgXDPAllowed: 123,
@@ -300,8 +303,8 @@ func (m *mockDataplane) ruleMatchID(dir rules.RuleDir, action string, owner rule
 	return h.Sum64()
 }
 
-func (m *mockDataplane) queryClassifier(ifindex, handle, prio int, ingress bool) (int, error) {
-	return 0, nil
+func (m *mockDataplane) queryClassifier(ifaceName, tcHook string) bool {
+	return true
 }
 
 var (
@@ -327,6 +330,19 @@ func (m *mockProgMapDP) loadPolicyProgram(progName string,
 	polProgsMap maps.Map,
 	opts ...polprog.Option,
 ) ([]fileDescriptor, []asm.Insns, error) {
+	if m.jitHarden {
+		builder := new(polprog.Builder)
+		for _, o := range opts {
+			o(builder)
+		}
+
+		if builder.TrampolineStride() > 15000 {
+			return nil, nil, unix.ERANGE
+		}
+
+		m.finalTrampolineStride = builder.TrampolineStride()
+	}
+
 	fdCounterLock.Lock()
 	defer fdCounterLock.Unlock()
 	fdCounter++
@@ -374,6 +390,7 @@ var _ = Describe("BPF Endpoint Manager", func() {
 		countersMap          *mock.Map
 		jumpMap              *mock.Map
 		xdpJumpMap           *mock.Map
+		qosMap               *mock.Map
 	)
 
 	BeforeEach(func() {
@@ -387,8 +404,6 @@ var _ = Describe("BPF Endpoint Manager", func() {
 		vxlanMTU = 0
 		nodePortDSR = true
 
-		bpfmaps.EnableRepin()
-
 		maps = new(bpfmap.Maps)
 
 		v4Maps = new(bpfmap.IPMaps)
@@ -404,6 +419,8 @@ var _ = Describe("BPF Endpoint Manager", func() {
 		commonMaps.StateMap = state.Map()
 		ifStateMap = mock.NewMockMap(ifstate.MapParams)
 		commonMaps.IfStateMap = ifStateMap
+		qosMap = mock.NewMockMap(qos.MapParams)
+		commonMaps.QoSMap = qosMap
 		cparams := counters.MapParameters
 		cparams.ValueSize *= bpfmaps.NumPossibleCPUs()
 		countersMap = mock.NewMockMap(cparams)
@@ -452,10 +469,6 @@ var _ = Describe("BPF Endpoint Manager", func() {
 		filterTableV6 = newMockTable("filter")
 	})
 
-	AfterEach(func() {
-		bpfmaps.DisableRepin()
-	})
-
 	newBpfEpMgr := func(ipv6Enabled bool) {
 		var err error
 		bpfEpMgr, err = NewBPFEndpointManager(
@@ -1044,6 +1057,24 @@ var _ = Describe("BPF Endpoint Manager", func() {
 		})
 	})
 
+	Context("with jit-harden", func() {
+		JustBeforeEach(func() {
+			dp.jitHarden = true
+			mockDP = &mockProgMapDP{
+				dp,
+			}
+			newBpfEpMgr(false)
+			genWLUpdate("cali12345")()
+			genIfaceUpdate("cali12345", ifacemonitor.StateUp, 15)()
+		})
+
+		It("should load program with shorter trampoline jumps", func() {
+			Expect(dp.programAttached("cali12345:ingress")).To(BeTrue())
+			Expect(dp.programAttached("cali12345:egress")).To(BeTrue())
+			Expect(dp.finalTrampolineStride).To(BeNumerically("<=", 15000))
+		})
+	})
+
 	Context("Ifacetype detection", func() {
 		JustBeforeEach(func() {
 			err := dp.createIface("vxlan0", 10, "vxlan")
