@@ -16,6 +16,7 @@ package ipsets_test
 
 import (
 	"fmt"
+	"slices"
 	"time"
 
 	. "github.com/onsi/ginkgo"
@@ -24,7 +25,7 @@ import (
 
 	"github.com/projectcalico/calico/felix/ip"
 	. "github.com/projectcalico/calico/felix/ipsets"
-	"github.com/projectcalico/calico/felix/labelindex"
+	"github.com/projectcalico/calico/felix/labelindex/ipsetmember"
 	"github.com/projectcalico/calico/felix/logutils"
 	"github.com/projectcalico/calico/felix/rules"
 	"github.com/projectcalico/calico/libcalico-go/lib/set"
@@ -75,7 +76,7 @@ var _ = Describe("IPSetTypeHashIPPort", func() {
 		Expect(CanonicaliseMember(IPSetTypeHashIPPort, "10.0.0.1,TCP:1234")).
 			To(Equal(V4IPPort{
 				IP:       ip.FromString("10.0.0.1").(ip.V4Addr),
-				Protocol: labelindex.ProtocolTCP,
+				Protocol: ipsetmember.ProtocolTCP,
 				Port:     1234,
 			}))
 	})
@@ -83,7 +84,7 @@ var _ = Describe("IPSetTypeHashIPPort", func() {
 		Expect(CanonicaliseMember(IPSetTypeHashIPPort, "10.0.0.1,SCTP:1234")).
 			To(Equal(V4IPPort{
 				IP:       ip.FromString("10.0.0.1").(ip.V4Addr),
-				Protocol: labelindex.ProtocolSCTP,
+				Protocol: ipsetmember.ProtocolSCTP,
 				Port:     1234,
 			}))
 	})
@@ -91,7 +92,7 @@ var _ = Describe("IPSetTypeHashIPPort", func() {
 		Expect(CanonicaliseMember(IPSetTypeHashIPPort, "feed:0::beef,uDp:3456")).
 			To(Equal(V6IPPort{
 				IP:       ip.FromString("feed::beef").(ip.V6Addr),
-				Protocol: labelindex.ProtocolUDP,
+				Protocol: ipsetmember.ProtocolUDP,
 				Port:     3456,
 			}))
 	})
@@ -179,14 +180,14 @@ var _ = Describe("IPPort types", func() {
 	It("V4 should stringify correctly", func() {
 		Expect(V4IPPort{
 			IP:       ip.FromString("10.0.0.0").(ip.V4Addr),
-			Protocol: labelindex.ProtocolTCP,
+			Protocol: ipsetmember.ProtocolTCP,
 			Port:     1234,
 		}.String()).To(Equal("10.0.0.0,tcp:1234"))
 	})
 	It("V6 should stringify correctly", func() {
 		Expect(V6IPPort{
 			IP:       ip.FromString("feed:beef::").(ip.V6Addr),
-			Protocol: labelindex.ProtocolUDP,
+			Protocol: ipsetmember.ProtocolUDP,
 			Port:     1234,
 		}.String()).To(Equal("feed:beef::,udp:1234"))
 	})
@@ -207,6 +208,7 @@ var _ = Describe("IPFamily", func() {
 var _ = Describe("IP sets dataplane", func() {
 	var dataplane *mockDataplane
 	var ipsets *IPSets
+	var listener *mockListener
 
 	meta := IPSetMetadata{
 		MaxSize: 1234,
@@ -248,7 +250,11 @@ var _ = Describe("IP sets dataplane", func() {
 
 	reschedRequested := false
 	apply := func() {
-		ipsets.ApplyUpdates()
+		if listener != nil { // Avoid nil interface/pointer confusion
+			ipsets.ApplyUpdates(listener)
+		} else {
+			ipsets.ApplyUpdates(nil)
+		}
 		reschedRequested = ipsets.ApplyDeletions()
 	}
 
@@ -258,6 +264,7 @@ var _ = Describe("IP sets dataplane", func() {
 	}
 
 	BeforeEach(func() {
+		listener = nil
 		dataplane = newMockDataplane()
 		ipsets = NewIPSetsWithShims(
 			v4VersionConf,
@@ -281,7 +288,7 @@ var _ = Describe("IP sets dataplane", func() {
 		// Apply updates.
 		ipsets.ApplyDeletions() // No-op
 		dataplane.ExpectMembers(map[string][]string{})
-		ipsets.ApplyUpdates()
+		ipsets.ApplyUpdates(nil)
 		dataplane.ExpectMembers(map[string][]string{
 			v4MainIPSetName: {"10.0.0.2", "10.0.0.3"},
 		})
@@ -684,8 +691,9 @@ var _ = Describe("IP sets dataplane", func() {
 			})
 		})
 
-		Describe("after creating second IP set", func() {
+		Describe("after creating second IP set, with a listener that filters to the main IP set", func() {
 			BeforeEach(func() {
+				listener = &mockListener{}
 				ipsets.AddOrReplaceIPSet(meta2, []string{"10.0.0.1", "10.0.0.3"})
 				apply()
 			})
@@ -697,6 +705,12 @@ var _ = Describe("IP sets dataplane", func() {
 				})
 			})
 
+			It("should record main IP set update only", func() {
+				ipsets.AddOrReplaceIPSet(meta, []string{"10.0.0.2", "10.0.0.4"})
+				apply()
+				Expect(listener.SeenMembers).To(Equal(set.From[string]("10.0.0.4")))
+			})
+
 			Context("with filtering to single IP set", func() {
 				BeforeEach(func() {
 					ipsets.SetFilter(set.From(v4MainIPSetName2))
@@ -741,9 +755,12 @@ var _ = Describe("IP sets dataplane", func() {
 				})
 			})
 
-			Describe("after another process flushes an IP set", func() {
+			Describe("after another process interferes with our IP sets", func() {
 				BeforeEach(func() {
+					// Flush one IP set.
 					dataplane.IPSetMembers[v4MainIPSetName] = set.New[string]()
+					// Remove one IP from the other IP set.
+					dataplane.IPSetMembers[v4MainIPSetName2] = set.From("10.0.0.1")
 				})
 
 				It("should be detected and fixed by a resync", func() {
@@ -753,6 +770,43 @@ var _ = Describe("IP sets dataplane", func() {
 						v4MainIPSetName2: {"10.0.0.1", "10.0.0.3"},
 					})
 				})
+
+				It("should only fix one IP set if problem is discovered as part of an update", func() {
+					// Simulate a failure to write one IP set.
+					dataplane.RestoreOpFailures = []string{"write-ip"}
+					// Trigger an update to only one IP set.
+					ipsets.AddMembers(ipSetID2, []string{"10.0.0.4"})
+
+					apply()
+					// The _other_ IP set shouldn't get fixed due to the
+					// targeted resync.
+					dataplane.ExpectMembers(map[string][]string{
+						v4MainIPSetName:  {},
+						v4MainIPSetName2: {"10.0.0.1", "10.0.0.3", "10.0.0.4"},
+					})
+
+					// But the next timer-triggered resync should catch it.
+					resyncAndApply()
+					dataplane.ExpectMembers(map[string][]string{
+						v4MainIPSetName:  {"10.0.0.1", "10.0.0.2"},
+						v4MainIPSetName2: {"10.0.0.1", "10.0.0.3", "10.0.0.4"},
+					})
+				})
+
+				It("should be detected after many transient errors", func() {
+					// Simulate lots of transient failures in a row, followed by success.
+					dataplane.RestoreOpFailures = slices.Repeat([]string{"write-ip"}, 6)
+					// Trigger an update to only one IP set.
+					ipsets.AddMembers(ipSetID2, []string{"10.0.0.4"})
+					apply()
+
+					// This time, the dataplane should escalate to a full resync,
+					// and both IP sets should get fixed.
+					dataplane.ExpectMembers(map[string][]string{
+						v4MainIPSetName:  {"10.0.0.1", "10.0.0.2"},
+						v4MainIPSetName2: {"10.0.0.1", "10.0.0.3", "10.0.0.4"},
+					})
+				})
 			})
 		})
 
@@ -802,7 +856,7 @@ var _ = Describe("IP sets dataplane", func() {
 			})
 			It("should panic eventually", func() {
 				ipsets.AddMembers(ipSetID, []string{"10.0.0.5"})
-				Expect(func() { ipsets.ApplyUpdates() }).To(Panic())
+				Expect(func() { ipsets.ApplyUpdates(nil) }).To(Panic())
 				Expect(dataplane.CumulativeSleep).To(BeNumerically(">", time.Second))
 			})
 		})
@@ -812,7 +866,7 @@ var _ = Describe("IP sets dataplane", func() {
 			})
 			It("should panic eventually", func() {
 				ipsets.QueueResync()
-				Expect(func() { ipsets.ApplyUpdates() }).To(Panic())
+				Expect(func() { ipsets.ApplyUpdates(nil) }).To(Panic())
 				Expect(dataplane.CumulativeSleep).To(BeNumerically(">", time.Second))
 			})
 		})
@@ -823,7 +877,7 @@ var _ = Describe("IP sets dataplane", func() {
 			})
 			It("should panic eventually", func() {
 				ipsets.QueueResync()
-				Expect(func() { ipsets.ApplyUpdates() }).To(Panic())
+				Expect(func() { ipsets.ApplyUpdates(nil) }).To(Panic())
 				Expect(dataplane.CumulativeSleep).To(BeNumerically(">", time.Second))
 			})
 		})
@@ -1073,3 +1127,18 @@ var _ = DescribeTable("ParseRange tests",
 	Entry("FOO-1", "FOO-1", 0, 0, true),
 	Entry("FOO", "FOO", 0, 0, true),
 )
+
+type mockListener struct {
+	SeenMembers set.Typed[string]
+}
+
+func (m *mockListener) CaresAboutIPSet(ipSetName string) bool {
+	return ipSetName == v4MainIPSetName
+}
+
+func (m *mockListener) OnMemberProgrammed(rawIPSetMember string) {
+	if m.SeenMembers == nil {
+		m.SeenMembers = set.New[string]()
+	}
+	m.SeenMembers.Add(rawIPSetMember)
+}
