@@ -323,9 +323,12 @@ func (s *IPSets) GetDesiredMembers(setID string) (set.Set[string], error) {
 	return strs, nil
 }
 
-// ApplyUpdates applies the updates to the dataplane.  Returns a set of programmed IPs in the IPSets included by the
-// ipsetFilter.
-func (s *IPSets) ApplyUpdates() {
+// ApplyUpdates applies the updates to the dataplane.  If listener is non-nil,
+// it will receive callbacks when members are programmed.  The callbacks occur
+// before we have final confirmation that the members are in the dataplane, so
+// the caller may wish to defer acting on the information until ApplyUpdates
+// returns.
+func (s *IPSets) ApplyUpdates(listener ipsets.UpdateListener) {
 	success := false
 	retryDelay := 1 * time.Millisecond
 	backOff := func() {
@@ -351,7 +354,7 @@ func (s *IPSets) ApplyUpdates() {
 			s.resyncRequired = false
 		}
 
-		if err := s.tryUpdates(); err != nil {
+		if err := s.tryUpdates(listener); err != nil {
 			// Update failures may mean that our iptables updates fail.  We need to do an immediate resync.
 			s.logCxt.WithError(err).Warning("Failed to update IP sets. Marking dataplane for resync.")
 			s.resyncRequired = true
@@ -580,7 +583,7 @@ func (s *IPSets) NFTablesSet(name string) *knftables.Set {
 }
 
 // tryUpdates attempts to apply any pending updates to the dataplane.
-func (s *IPSets) tryUpdates() error {
+func (s *IPSets) tryUpdates(listener ipsets.UpdateListener) error {
 	var dirtyIPSets []string
 
 	s.ipSetsWithDirtyMembers.Iter(func(setName string) error {
@@ -618,8 +621,8 @@ func (s *IPSets) tryUpdates() error {
 	for _, setName := range dirtyIPSets {
 		// If the set is already programmed, we can skip it.
 		if _, ok := s.setNameToProgrammedMetadata.Dataplane().Get(setName); !ok {
-			if set := s.NFTablesSet(setName); set != nil {
-				tx.Add(set)
+			if nfSet := s.NFTablesSet(setName); nfSet != nil {
+				tx.Add(nfSet)
 			}
 		}
 
@@ -634,10 +637,14 @@ func (s *IPSets) tryUpdates() error {
 		})
 
 		// Add desired members to the set.
+		listenerCares := listener != nil && listener.CaresAboutIPSet(setName)
 		members.Desired().Iter(func(member SetMember) {
 			if members.Dataplane().Contains(member) {
 				return
 			}
+			if listenerCares {
+				listener.OnMemberProgrammed(member.String())
+			}
 			tx.Add(&knftables.Element{
 				Set: setName,
 				Key: member.Key(),
@@ -805,6 +812,7 @@ func CanonicaliseMember(t ipsets.IPSetType, member string) SetMember {
 		if ipAddr == nil {
 			// This should be prevented by validation in libcalico-go.
 			log.WithField("ip", member).Panic("Failed to parse IP")
+			panic("Failed to parse IP part of IP,port member")
 		}
 		return simpleMember(ipAddr.String())
 	case ipsets.IPSetTypeHashIPPort:
@@ -817,6 +825,7 @@ func CanonicaliseMember(t ipsets.IPSetType, member string) SetMember {
 		if ipAddr == nil {
 			// This should be prevented by validation.
 			log.WithField("member", member).Panic("Failed to parse IP part of IP,port member")
+			panic("Failed to parse IP part of IP,port member")
 		}
 		parts = strings.Split(parts[1], ":")
 		if len(parts) != 2 {
