@@ -16,7 +16,6 @@ package clientv3_test
 
 import (
 	"context"
-	"os/exec"
 	"time"
 
 	. "github.com/onsi/ginkgo"
@@ -25,10 +24,12 @@ import (
 	apiv3 "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
 	v3 "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	ctrlclient "sigs.k8s.io/controller-runtime/pkg/client"
 
 	"github.com/projectcalico/calico/libcalico-go/lib/apiconfig"
 	"github.com/projectcalico/calico/libcalico-go/lib/backend"
 	bapi "github.com/projectcalico/calico/libcalico-go/lib/backend/api"
+	"github.com/projectcalico/calico/libcalico-go/lib/backend/k8s"
 	"github.com/projectcalico/calico/libcalico-go/lib/clientv3"
 	"github.com/projectcalico/calico/libcalico-go/lib/options"
 	"github.com/projectcalico/calico/libcalico-go/lib/testutils"
@@ -402,10 +403,25 @@ var _ = testutils.E2eDatastoreDescribe("StagedGlobalNetworkPolicy tests", testut
 	Describe("StagedGlobalNetworkPolicy without name on the projectcalico.org annotation", func() {
 		It("Should return the name without default prefix", func() {
 			if config.Spec.DatastoreType == apiconfig.Kubernetes {
-				// We create the policies as a CRD to prevent the api server adding the correct annotation
-				err := exec.Command("kubectl", "create", "-f", "../../test/mock-policies.yaml").Run()
-				Expect(err).ToNot(HaveOccurred())
+				config, _, err := k8s.CreateKubernetesClientset(&config.Spec)
+				Expect(err).NotTo(HaveOccurred())
+				config.ContentType = "application/json"
+				cli, err := ctrlclient.New(config, ctrlclient.Options{})
+				Expect(err).NotTo(HaveOccurred())
 
+				// Create v1 crd with empty metadata annotation name
+				annotations := map[string]string{}
+				annotations["projectcalico.org/metadata"] = "{}"
+				policy := &apiv3.StagedGlobalNetworkPolicy{
+					ObjectMeta: metav1.ObjectMeta{
+						Annotations: annotations,
+						Name:        "default.prefix-test-policy"},
+					Spec: apiv3.StagedGlobalNetworkPolicySpec{},
+				}
+				err = cli.Create(context.Background(), policy)
+				Expect(err).NotTo(HaveOccurred())
+
+				// We should be able to get it without the default. prefix
 				_, err = c.StagedGlobalNetworkPolicies().Get(ctx, "prefix-test-policy", options.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 			}
@@ -599,7 +615,7 @@ var _ = testutils.E2eDatastoreDescribe("StagedGlobalNetworkPolicy tests", testut
 
 			By("Cleaning the datastore and expecting deletion events for each configured resource (tests prefix deletes results in individual events for each key)")
 			be.Clean()
-			testWatcher4.ExpectEvents(apiv3.KindStagedGlobalNetworkPolicy, []watch.Event{
+			testWatcher4.ExpectEventsAnyOrder(apiv3.KindStagedGlobalNetworkPolicy, []watch.Event{
 				{
 					Type:     watch.Deleted,
 					Previous: outRes1,
