@@ -6,6 +6,7 @@ import (
 	"testing"
 
 	"github.com/projectcalico/calico/release/internal/utils"
+	"github.com/projectcalico/calico/release/pkg/manager/operator"
 )
 
 var (
@@ -46,4 +47,9 @@ func checkImages(t testing.TB, images string) {
 	if len(list) == 0 {
 		t.Fatal("No images provided")
 	}
+	for _, image := range list {
+		if strings.Contains(image, operator.DefaultImage) {
+			t.Fatal("Operator images are checked separately, do not include in list of images to check")
+		}
+	}
 }
