@@ -15,6 +15,8 @@
 package tasks
 
 import (
+	"github.com/sirupsen/logrus"
+
 	"github.com/projectcalico/calico/release/internal/hashreleaseserver"
 	"github.com/projectcalico/calico/release/internal/slack"
 	"github.com/projectcalico/calico/release/internal/utils"
@@ -24,6 +26,7 @@ var product = utils.ProductName
 
 // AnnounceHashrelease sends a slack notification for a new hashrelease.
 func AnnounceHashrelease(cfg *slack.Config, hashrel *hashreleaseserver.Hashrelease, ciURL string) error {
+	logrus.WithField("hashrelease", hashrel.Name).Info("Sending hashrelease announcement to Slack")
 	msgData := &slack.HashreleaseMessageData{
 		ReleaseName:        hashrel.Name,
 		Product:            product,
